@@ -6,7 +6,7 @@ hwe.hardy<-function(a,alleles=3,seed=3000,sample=c(1000,1000,5000))
   swp<-rep(0,3)
   z<-.C("hwe_hardy",a=as.integer(a), alleles=as.integer(alleles), 
         seed=as.integer(seed), gss=as.integer(sample),
-        p=as.double(p), se=as.double(se), swp=as.double(swp))
+        p=as.double(p), se=as.double(se), swp=as.double(swp),PACKAGE="gap")
 
   list(p=z$p, se=z$se, swp=z$swp)
 }
